;*****************************************************************************
;				NICST TOOL
;
;  FU1 RC-5 Part 1 test for TEB bands. Checks the gain switching for M13.
;  Use only the auto gain collects for M13.
;
;  Input:
;  1) File with inpath and outpath.
;
;  Output:
;  1) dn_1[collect,band,ham,det,subset,gain,frame]
;  2) snr[collect,band,ham,det,subset,gain,frame]
;  3) max_dn_frame[collect,band,ham,det,subset,scan,0=up 1=down]
;  4) max_dn_scan[collect,band,ham,det,subset,0=up 1=down]
;  5) max_dn[band,ham,det,subset,0=up 1=down]
;  6) max_L[band,ham,det,subset,0=up 1=down]
;  7) max_temp[band,ham,det,subset,0=up 1=down]
;  8) transition_slope_up[collect,band,ham,det,subset,scan,0=high 1=low]
;  9) transition_slope_down[collect,band,ham,det,subset,scan,0=high 1=low]
;  
;  v1.0 - 02/25/2009 - Jeff McIntire
;*****************************************************************************

PRO RC5_TEB_Gain_dn_3, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and get list of dump files.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.
fnumber=strarr(1)
header_size=intarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
readf,lun,header_size
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

restore,'general.TEB.sav'
restore,'gain_M13_hg.sav'

; Get list of .dmp files from specified path. 
spawn, 'ls -C1 ' + inpath + '*M13*.dmp', list_files
IF N_ELEMENTS(list_files) EQ 0 THEN goto,skipout
nfile=N_ELEMENTS(list_files)
ncollects=nfile/2

;*****************************************************************************
;  Find the maximum number of frames.
;*****************************************************************************

FOR ifile=0, nfile-1 DO BEGIN

    nlines=File_Lines(list_files[ifile])
    IF nlines EQ 0 THEN continue
    
    a2=strsplit(list_files[ifile],'/',/Extract)
    a3=N_ELEMENTS(a2)
    a8=strmid(a2[a3-1],strpos(a2[a3-1],'/',/Reverse_Search)+10,2) ; Determine EV or SV.

    ; Keep only the EV.
    IF a8 EQ 'SV' OR a8 EQ 'OB' THEN continue
    
    read_dmpd2,list_files[ifile],header_size,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns
    max_frame=max(frame)-min(frame)+1

    break
ENDFOR

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

max_scan=40					; Total # scans.
minframe=intarr(ncollects)
maxframe=intarr(ncollects)

outfile=' '					; Output file name, which is reasigned.

; dn.
dn_col=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxgain,max_scan,2)
dn_1=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxgain,max_frame)

; Standard deviation
std_1=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxgain,max_frame)

; SNR
snr=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxgain,max_frame)

; Transistion slopes
transition_slope_up=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan,2)
transition_slope_down=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan,2)

; Max dn, L, and temperature.
max_dn_frame=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan,2)
low_dn_frame=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan,2)
max_dn_scan=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,2)
low_dn_scan=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,2)
max_dn=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2)
low_dn=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2)
max_L=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2)
low_L=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2)
max_temp=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2)
low_temp=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2)

;*****************************************************************************
;  Read in the dump files for SV.
;*****************************************************************************

a5=-1
FOR ifile=0, nfile-1 DO BEGIN

    nlines=File_Lines(list_files[ifile])
    IF nlines EQ 0 THEN continue
    
    a2=strsplit(list_files[ifile],'/',/Extract)
    a3=N_ELEMENTS(a2)
    a8=strmid(a2[a3-1],strpos(a2[a3-1],'/',/Reverse_Search)+10,2) ; Determine EV or SV.
    
    ; Keep only the SV.
    IF a8 EQ 'EV' THEN continue
    IF a8 EQ 'OB' THEN continue
    a5=a5+1
    print,list_files[ifile]
  
    read_dmpd2,list_files[ifile],header_size,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns

    ; Translate the band name to a number.
    b=3
    
    max_frame=48 ; Total # frames (M band).
    
;*****************************************************************************
;  Write DN values to an array for SV.
;*****************************************************************************

    dn_frame=dblarr(nham,maxdet,maxsubsets,maxgain,max_scan,max_frame)
    iscan=0
    idet=0
    iframe=0
    subset=0
    FOR i=0L, nlines-1 DO BEGIN
        IF scan[i] EQ iscan+1 AND det[i] EQ idet+1 THEN BEGIN
            dn_frame[ham[i],idet,subset,gain_type[i],iscan,iframe]=dns[i]
            iframe=iframe+1
            IF iframe EQ max_frame THEN BEGIN
                idet=idet+1
                iframe=0
                IF idet EQ detectors_TEB[b] THEN BEGIN
                    idet=0
                    iscan=iscan+1
                ENDIF
            ENDIF
        ENDIF
        IF band_type_TEB[b] EQ 1 THEN BEGIN
            IF subset EQ 0 THEN BEGIN
                subset=1
                goto,skip_over_1
            ENDIF
            IF subset EQ 1 THEN subset=0
            skip_over_1:
        ENDIF
    ENDFOR

;*****************************************************************************
;  Average over frames and calculate the standard deviation.
;*****************************************************************************

    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    FOR iscan=0, max_scan-1 DO BEGIN
                        dn_col[a5,b,iham,idet,isubset,igain,iscan,1]= $
                          sig_mean_no_kick(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],1)
                        
                        IF dn_col[a5,b,iham,idet,isubset,igain,iscan,1] EQ -999 THEN $
                          dn_col[a5,b,iham,idet,isubset,igain,iscan,1]=0
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Determine if auto gain or fixed high gain for M13, detector 1 using
;  the first four scans of collect 1.
;*****************************************************************************

IF dn_col[0,3,0,0,0,0,0,1] EQ 0 THEN j1=0 ELSE j1=1
IF dn_col[0,3,0,0,0,0,1,1] EQ 0 THEN j2=0 ELSE j2=1
IF dn_col[0,3,0,0,0,0,2,1] EQ 0 THEN j3=0 ELSE j3=1
IF dn_col[0,3,0,0,0,0,3,1] EQ 0 THEN j4=0 ELSE j4=1

; if all zero then fixed high gain, if one non-zero then suto gain,
; and if two non-zero then fixed low gain.
IF j1+j2+j3+j4 EQ 0 THEN type_gain=0
IF j1+j2+j3+j4 EQ 1 THEN type_gain=1
IF j1+j2+j3+j4 EQ 2 THEN type_gain=2

; Auto Gain: write low background to index with high background and vice versa.
IF type_gain EQ 1 THEN BEGIN
    FOR col=0, ncollects-1 DO BEGIN
        FOR iham=0, nham-1 DO BEGIN
            FOR idet=0, detectors_TEB[b]-1 DO BEGIN
                FOR iscan=0, max_scan-1 DO BEGIN
                    IF iscan+4 LT max_scan-1 THEN BEGIN
                        IF dn_col[col,3,iham,idet,0,0,iscan,1] GT 0 AND dn_col[col,3,iham,idet,0,0,iscan+4,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,0,iscan+2,1]=dn_col[col,3,iham,idet,0,0,iscan,1]
                        IF dn_col[col,3,iham,idet,0,1,iscan,1] GT 0 AND dn_col[col,3,iham,idet,0,1,iscan+4,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,1,iscan+2,1]=dn_col[col,3,iham,idet,0,1,iscan,1]
                    ENDIF
                    IF iscan LE 1 THEN BEGIN
                        IF dn_col[col,3,iham,idet,0,0,iscan+2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,0,iscan,1]=dn_col[col,3,iham,idet,0,0,iscan+2,1]
                        IF dn_col[col,3,iham,idet,0,1,iscan+2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,1,iscan,1]=dn_col[col,3,iham,idet,0,1,iscan+2,1]
                    ENDIF
                    IF iscan GE max_scan-3 THEN BEGIN
                        IF dn_col[col,3,iham,idet,0,0,iscan-2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,0,iscan,1]=dn_col[col,3,iham,idet,0,0,iscan-2,1]
                        IF dn_col[col,3,iham,idet,0,1,iscan-2,1] GT 0 THEN $
                          dn_col[col,3,iham,idet,0,1,iscan,1]=dn_col[col,3,iham,idet,0,1,iscan-2,1]
                    ENDIF
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDIF

;*****************************************************************************
;  Read in the dump files for EV.
;*****************************************************************************

a5=-1
FOR ifile=0, nfile-1 DO BEGIN

    nlines=File_Lines(list_files[ifile])
    IF nlines EQ 0 THEN continue
    
    a2=strsplit(list_files[ifile],'/',/Extract)
    a3=N_ELEMENTS(a2)
    a8=strmid(a2[a3-1],strpos(a2[a3-1],'/',/Reverse_Search)+10,2) ; Determine EV or SV.

    ; Keep only the EV.
    IF a8 EQ 'SV' OR a8 EQ 'OB' THEN continue
    a5=a5+1
    print,list_files[ifile]
    
    read_dmpd2,list_files[ifile],header_size,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns

    ; Translate the band name to a number.
    b=3
    
    max_frame=max(frame)-min(frame)+1
    minframe[a5]=min(frame)
    maxframe[a5]=max(frame)

;*****************************************************************************
;  Write DN values to an array for EV.
;*****************************************************************************

    dn_frame=dblarr(nham,maxdet,maxsubsets,maxgain,max_scan,max_frame)
    iscan=0
    idet=0
    iframe=0
    subset=0
    FOR i=0L, nlines-1 DO BEGIN
        IF scan[i] EQ iscan+1 AND det[i] EQ idet+1 THEN BEGIN
            dn_frame[ham[i],idet,subset,gain_type[i],iscan,iframe]=dns[i]
            iframe=iframe+1
            IF iframe EQ max_frame THEN BEGIN
                idet=idet+1
                iframe=0
                IF idet EQ detectors_TEB[b] THEN BEGIN
                    idet=0
                    iscan=iscan+1
                ENDIF
            ENDIF
        ENDIF
        IF band_type_TEB[b] EQ 1 THEN BEGIN
            IF subset EQ 0 THEN BEGIN
                subset=1
                goto,skip_over_2
            ENDIF
            IF subset EQ 1 THEN subset=0
            skip_over_2:
        ENDIF
    ENDFOR

;*****************************************************************************
;  Subtract the SV average over frames from each frame of EV (keep label of
;  first subsample as 0 and second subsample as 1).
;*****************************************************************************

    dn_col_1=dblarr(nham,maxdet,maxgain,maxsubsets,max_scan,max_frame)
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    FOR iscan=0, max_scan-1 DO BEGIN
                        FOR iframe=0, max_frame-1 DO BEGIN
                            IF dn_frame[iham,idet,isubset,igain,iscan,iframe] NE 0 THEN $
                              dn_col_1[iham,idet,isubset,igain,iscan,iframe]= $
                              dn_frame[iham,idet,isubset,igain,iscan,iframe] $
                              -dn_col[a5,b,iham,idet,isubset,igain,iscan,1]
                            IF dn_frame[iham,idet,isubset,igain,iscan,iframe] EQ 0 THEN $
                              dn_col_1[iham,idet,isubset,igain,iscan,iframe]=0
                        ENDFOR
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
        
;*****************************************************************************
;  Find the dn and standard deviation over scans for each frame.
;  Then determine the SNR.
;*****************************************************************************

    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR igain=0, 1 DO BEGIN
                    FOR iframe=0, max_frame-1 DO BEGIN
                        IF mean(dn_col_1[iham,idet,isubset,igain,0:max_scan-1,iframe]) NE 0 THEN BEGIN
                            dn_1[a5,b,iham,idet,isubset,igain,iframe]= $
                              sig_mean_no_kick(dn_col_1[iham,idet,isubset,igain,0:max_scan-1,iframe],1)
                            std_1[a5,b,iham,idet,isubset,igain,iframe]= $
                              sig_mean_no_kick(dn_col_1[iham,idet,isubset,igain,0:max_scan-1,iframe],2)
                            IF std_1[a5,b,iham,idet,isubset,igain,iframe] NE 0 THEN BEGIN
                                snr[a5,b,iham,idet,isubset,igain,iframe]=dn_1[a5,b,iham,idet,isubset,igain,iframe] $
                                  /std_1[a5,b,iham,idet,isubset,igain,iframe]
                            ENDIF
                        ENDIF ELSE BEGIN
                            dn_1[a5,b,iham,idet,isubset,igain,iframe]=0
                            std_1[a5,b,iham,idet,isubset,igain,iframe]=0
                        ENDELSE
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR

;*****************************************************************************
;  Find the high gain to low gain transition. Find the slope of the
;  transition and find the maximum high to low dn,
;*****************************************************************************

    y=findgen(6)
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR iscan=0, max_scan-1 DO BEGIN
                    FOR iframe=0, max_frame-2 DO BEGIN
                        IF dn_frame[iham,idet,isubset,1,iscan,iframe] NE 0 AND dn_frame[iham,idet,isubset,0,iscan,iframe+1] NE 0 THEN BEGIN 

                            ; Slope of transition
                            result1=poly_fit(y[0:5],dn_frame[iham,idet,isubset,1,iscan,iframe-5:iframe],1)
                            result2=poly_fit(y[0:5],dn_frame[iham,idet,isubset,0,iscan,iframe+1:iframe+6],1)

                            transition_slope_up[a5,b,iham,idet,isubset,iscan,0]=result1[1]
                            transition_slope_up[a5,b,iham,idet,isubset,iscan,1]=result2[1]

                            ; Maximum dn.
                            max_dn_frame[a5,b,iham,idet,isubset,iscan,0]=max(dn_col_1[iham,idet,isubset,1,iscan,0:iframe])
                            low_dn_frame[a5,b,iham,idet,isubset,iscan,0]=dn_col_1[iham,idet,isubset,0,iscan,iframe+1]
                        ENDIF
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR

;*****************************************************************************
;  Find the low gain to high gain transition. Find the slope of the
;  transition and find the maximum low to high dn,
;*****************************************************************************

    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                FOR iscan=0, max_scan-1 DO BEGIN
                    FOR iframe=0, max_frame-2 DO BEGIN
                        IF dn_frame[iham,idet,isubset,0,iscan,iframe] NE 0 AND dn_frame[iham,idet,isubset,1,iscan,iframe+1] NE 0 THEN BEGIN

                            ; Slope of transition
                            result1=poly_fit(y[0:5],dn_frame[iham,idet,isubset,0,iscan,iframe-5:iframe],1)
                            result2=poly_fit(y[0:5],dn_frame[iham,idet,isubset,1,iscan,iframe+1:iframe+6],1)

                            transition_slope_down[a5,b,iham,idet,isubset,iscan,0]=result1[1]
                            transition_slope_down[a5,b,iham,idet,isubset,iscan,1]=result2[1]

                            ; Maximum dn.
                            max_dn_frame[a5,b,iham,idet,isubset,iscan,1]=max(dn_col_1[iham,idet,isubset,1,iscan,iframe+1:max_frame-1])
                            low_dn_frame[a5,b,iham,idet,isubset,iscan,1]=dn_col_1[iham,idet,isubset,0,iscan,iframe]
                        ENDIF
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR

;*****************************************************************************
;  Find the maximum high gain dn over scans.
;*****************************************************************************

    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                max_dn_scan[a5,b,iham,idet,isubset,0]=max(max_dn_frame[a5,b,iham,idet,isubset,0:max_scan-1,0])
                max_dn_scan[a5,b,iham,idet,isubset,1]=max(max_dn_frame[a5,b,iham,idet,isubset,0:max_scan-1,1])
                low_dn_scan[a5,b,iham,idet,isubset,0]=sig_mean_no_kick(low_dn_frame[a5,b,iham,idet,isubset,0:max_scan-1,0],1)
                low_dn_scan[a5,b,iham,idet,isubset,1]=sig_mean_no_kick(low_dn_frame[a5,b,iham,idet,isubset,0:max_scan-1,1],1)

                IF low_dn_scan[a5,b,iham,idet,isubset,0] EQ -999. THEN low_dn_scan[a5,b,iham,idet,isubset,0]=0.
                IF low_dn_scan[a5,b,iham,idet,isubset,1] EQ -999. THEN low_dn_scan[a5,b,iham,idet,isubset,1]=0.
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Find the maximum high gain dn over collects.
;*****************************************************************************

HAM=['A','B']
FOR iham=0, nham-1 DO BEGIN
    FOR idet=0, detectors_TEB[b]-1 DO BEGIN
        FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
            max_dn[3,iham,idet,isubset,0]=max(max_dn_scan[0:ncollects-1,3,iham,idet,isubset,0])
            max_dn[3,iham,idet,isubset,1]=max(max_dn_scan[0:ncollects-1,3,iham,idet,isubset,1])
            low_dn[3,iham,idet,isubset,0]=sig_mean_no_kick(low_dn_scan[0:ncollects-1,3,iham,idet,isubset,0],1)
            low_dn[3,iham,idet,isubset,1]=sig_mean_no_kick(low_dn_scan[0:ncollects-1,3,iham,idet,isubset,1],1)
            IF iham EQ 0 THEN print,'Max dn for detector '+strtrim(string(idet+1),2)+' HAM ='+HAM[iham]+' ,', $
              max_dn[3,iham,idet,isubset,0],max_dn[3,iham,idet,isubset,1]
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Convert maximum dn to radiance (divide by gain).
;*****************************************************************************

FOR iham=0, nham-1 DO BEGIN
    FOR idet=0, detectors_TEB[b]-1 DO BEGIN
        FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
            max_L[3,iham,idet,isubset,0]=max_dn[3,iham,idet,isubset,0]/gain[3,iham,idet,isubset,1]
            max_L[3,iham,idet,isubset,1]=max_dn[3,iham,idet,isubset,1]/gain[3,iham,idet,isubset,1]
            low_L[3,iham,idet,isubset,0]=low_dn[3,iham,idet,isubset,0]/gain[3,iham,idet,isubset,0]
            low_L[3,iham,idet,isubset,1]=low_dn[3,iham,idet,isubset,1]/gain[3,iham,idet,isubset,0]
            IF iham EQ 0 THEN print,'Max radiance for detector '+strtrim(string(idet+1),2)+' HAM ='+HAM[iham]+' ,', $
              max_L[3,iham,idet,isubset,0],max_L[3,iham,idet,isubset,1]
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Convert maximum radiance to temperature.
;*****************************************************************************

c1=119106211.8D
c2=14387.86D
cwl=4.050

FOR iham=0, nham-1 DO BEGIN
    FOR idet=0, detectors_TEB[b]-1 DO BEGIN
        FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
            max_temp[3,iham,idet,isubset,0]=c2/cwl/alog(c1/cwl^5/max_L[3,iham,idet,isubset,0]+1.)
            max_temp[3,iham,idet,isubset,1]=c2/cwl/alog(c1/cwl^5/max_L[3,iham,idet,isubset,1]+1.)
            low_temp[3,iham,idet,isubset,0]=c2/cwl/alog(c1/cwl^5/low_L[3,iham,idet,isubset,0]+1.)
            low_temp[3,iham,idet,isubset,1]=c2/cwl/alog(c1/cwl^5/low_L[3,iham,idet,isubset,1]+1.)
            IF iham EQ 0 THEN print,'Max temperature for detector '+strtrim(string(idet+1),2)+' HAM ='+HAM[iham]+' ,', $
              max_temp[3,iham,idet,isubset,0],max_temp[3,iham,idet,isubset,1]
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Make a sav file
;*****************************************************************************

save,/compress,dn_col_1,snr,max_dn_frame,max_dn_scan,max_dn,max_L,max_temp,low_dn_frame,low_dn_scan,low_dn,low_L,low_temp, $
  transition_slope_up,transition_slope_down,filename=outpath+'.dn.sav'

;*****************************************************************************
;  Set max high gain dn plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outpath+'.max_dn.ps'
device,/landscape,filename=outfname,/color

D=findgen(4000)
HAM=['A','B']
GAIN=['Low gain','High gain']

;*****************************************************************************
;  Plot max high gain dn vs scan.
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
    title='Max high gain dn vs Scan '+band_name_TEB[3]+' Collect '+strtrim(string(col+1),2)
    FOR iham=0, nham-1 DO BEGIN
        FOR isubset=0, subsets_TEB[3]-1 DO BEGIN
            FOR j=0, 1 DO BEGIN
                IF mean(max_dn_frame[col,3,iham,0,isubset,0:max_scan-1,j]) EQ 0 THEN continue

                IF j EQ 0 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (high to low)'
                IF j EQ 1 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (low to high)'

                zz=WHERE(max_dn_frame[col,3,iham,0,isubset,0:max_scan-1,j] NE 0)

                IF mean(max_dn_frame[col,3,iham,0,isubset,zz,j]) LE 0 THEN continue

                ymax=max(max_dn_frame[col,3,iham,0:detectors_TEB[3]-1,isubset,zz,j],min=ymin)
                
                plot,D[0:max_scan/2-1]+1+zz[0],max_dn_frame[col,3,iham,0,isubset,zz,j],xtit='Scan',ytit='Max high gain dn',xrange=[0,max_scan], $
                  yrange=[ymin,ymax],title=title2,charsize=1.2,charthick=5,thick=5,/nodata
    
                FOR idet=0, detectors_TEB[b]-1 DO BEGIN
                    oplot,2*D[0:max_scan/2-1]+1+zz[0],max_dn_frame[col,3,iham,idet,isubset,zz,j],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                    oplot,2*D[0:max_scan/2-1]+1+zz[0],max_dn_frame[col,3,iham,idet,isubset,zz,j],symsize=1,color=lcol[idet],thick=5
                ENDFOR
                XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                set_legend,1,chname,lcol,symbol
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Plot max high gain dn vs collect.
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

title='Max high gain dn vs Collect '+band_name_TEB[3]
FOR iham=0, nham-1 DO BEGIN
    FOR isubset=0, subsets_TEB[3]-1 DO BEGIN
        FOR j=0, 1 DO BEGIN
            IF mean(max_dn_scan[0:ncollects-1,3,iham,0,isubset,j]) LE 0 THEN continue

            IF j EQ 0 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (high to low)'
            IF j EQ 1 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (low to high)'
        
            ymax=max(max_dn_scan[0:ncollects-1,3,iham,0:detectors_TEB[3]-1,isubset,j],min=ymin)
        
            plot,D[0:ncollects-1]+1,max_dn_scan[0:ncollects-1,3,iham,0,isubset,j],xtit='Collect',ytit='Max high gain dn',xrange=[0,ncollects], $
              yrange=[ymin,ymax],title=title2,charsize=1.2,charthick=5,thick=5,/nodata
        
            FOR idet=0, detectors_TEB[b]-1 DO BEGIN
                oplot,D[0:ncollects-1]+1,max_dn_scan[0:ncollects-1,3,iham,idet,isubset,j],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                oplot,D[0:ncollects-1]+1,max_dn_scan[0:ncollects-1,3,iham,idet,isubset,j],symsize=1,color=lcol[idet],thick=5
            ENDFOR
            XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
            set_legend,1,chname,lcol,symbol
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Plot max high gain temperature vs detector
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

title='Gain Switching Temperature vs Detector '+band_name_TEB[3]
FOR iham=0, nham-1 DO BEGIN
    FOR isubset=0, subsets_TEB[3]-1 DO BEGIN
        FOR j=0, 1 DO BEGIN
            IF mean(max_temp[3,iham,0:detectors_TEB[3]-1,isubset,j]) LE 0 THEN continue
        
            IF j EQ 0 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (high to low)'
            IF j EQ 1 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (low to high)'

            ymax=max(max_temp[3,iham,0:detectors_TEB[3]-1,isubset,j],min=ymin)
            IF ymax LT 350. THEN ymax=350.
            IF ymin GT 340. THEN ymin=340.
            
            plot,D[0:detectors_TEB[3]-1]+1,max_temp[3,iham,0:detectors_TEB[3]-1,isubset,j],xtit='Detector',ytit='T (K)',yrange=[ymin,ymax],title=title2, $
              charsize=1.2,charthick=5,thick=5,/nodata
            
            oplot,D[0:detectors_TEB[3]-1]+1,max_temp[3,iham,0:detectors_TEB[3]-1,isubset,j],symsize=1,color=lcol[0],psym=symbol[0],thick=5
            oplot,D[0:detectors_TEB[3]-1]+1,max_temp[3,iham,0:detectors_TEB[3]-1,isubset,j],symsize=1,color=lcol[0],thick=5
            
            oplot,[0.,40.],[343.,343.],symsize=1,color=lcol[1],linestyle=2,thick=5
            oplot,[0.,40.],[348.,348.],symsize=1,color=lcol[1],linestyle=2,thick=5
            
            XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Plot SNR and standard deviaiton vs frame per collect
;*****************************************************************************

; Reform the SNR
lowhigh=intarr(ncollects,2)
highlow=intarr(ncollects,2)
snr_1=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_frame)
std=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_frame)
FOR collect=0, ncollects-1 DO BEGIN
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[b]-1 DO BEGIN
            FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
                k=0
                FOR iframe=0, max_frame-1 DO BEGIN
                    IF snr[collect,b,iham,idet,isubset,1,iframe] NE 0 THEN BEGIN
                        snr_1[collect,b,iham,idet,isubset,iframe]=snr[collect,b,iham,idet,isubset,1,iframe]
                        std[collect,b,iham,idet,isubset,iframe]=std_1[collect,b,iham,idet,isubset,1,iframe]
                    ENDIF
                    IF snr[collect,b,iham,idet,isubset,1,iframe] EQ 0 THEN BEGIN
                        snr_1[collect,b,iham,idet,isubset,iframe]=snr[collect,b,iham,idet,isubset,0,iframe]
                        std[collect,b,iham,idet,isubset,iframe]=std_1[collect,b,iham,idet,isubset,0,iframe]
                    ENDIF
                    IF idet/2-idet/2.+.5 NE 0 THEN x=0 ELSE x=1
                    IF dn_1[collect,b,iham,idet,isubset,0,iframe] NE 0 AND dn_1[collect,b,iham,idet,isubset,1,iframe] EQ 0 THEN lowhigh[collect,x]=iframe+1
                    IF k EQ 0 THEN BEGIN
                        IF dn_1[collect,b,iham,idet,isubset,0,iframe] NE 0 AND dn_1[collect,b,iham,idet,isubset,1,iframe] EQ 0 THEN highlow[collect,x]=iframe-1
                        IF dn_1[collect,b,iham,idet,isubset,0,iframe] NE 0 AND dn_1[collect,b,iham,idet,isubset,1,iframe] EQ 0 THEN k=1
                    ENDIF
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

!p.multi=[0,1,2]
!p.position=0

E=fltarr(ncollects,2,max_frame)
FOR collect=0, ncollects-1 DO BEGIN
    FOR odd_even=0, 1 DO BEGIN
        FOR iframe=0, max_frame-1 DO BEGIN
            IF iframe LE highlow[collect,odd_even] THEN E[collect,odd_even,iframe]=100.
            IF iframe GE lowhigh[collect,odd_even] THEN E[collect,odd_even,iframe]=100.
            IF iframe GT highlow[collect,odd_even] AND iframe LT lowhigh[collect,odd_even] THEN E[collect,odd_even,iframe]=10.
        ENDFOR
    ENDFOR
ENDFOR

even=[1,3,5,7,9,11,13,15]
odd=[0,2,4,6,8,10,12,14]
oddeven=['ODD','EVEN']

FOR collect=0, ncollects-1 DO BEGIN
    title=band_name_TEB[3]+' SNR vs frame, Collect '+strtrim(string(collect+1),2)
    FOR iham=0, nham-1 DO BEGIN
        FOR isubset=0, subsets_TEB[3]-1 DO BEGIN
            IF mean(snr_1[collect,3,iham,0,isubset,0:max_frame-1]) LE 0 THEN continue

            FOR odd_even=0, 1 DO BEGIN
        
                title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' '+oddeven[odd_even]

                IF odd_even EQ 0 THEN x=odd ELSE x=even
                ymax=max(snr_1[collect,3,iham,x,isubset,0:max_frame-1])
                ymin=1.

                plot,D[0:max_frame-1]+1,snr_1[collect,3,iham,0,isubset,0:max_frame-1],xtit='frame',ytit='SNR',xrange=[minframe[collect],maxframe[collect]], $
                  yrange=[ymin,ymax],title=title2,charsize=1.2,charthick=5,thick=5,/nodata,/ylog
            
                IF odd_even EQ 0 THEN y=0 ELSE y=1
                FOR idet=y, detectors_TEB[b]-1, 2 DO BEGIN
                    oplot,D[0:max_frame-1]+minframe[collect],snr_1[collect,3,iham,idet,isubset,0:max_frame-1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                    oplot,D[0:max_frame-1]+minframe[collect],snr_1[collect,3,iham,idet,isubset,0:max_frame-1],symsize=1,color=lcol[idet],thick=5
                ENDFOR
                oplot,D[0:max_frame-1]+minframe[collect],E[collect,odd_even,0:max_frame-1],symsize=1,color=lcol[1],thick=5

                XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

                ymax=max(std[collect,3,iham,x,isubset,0:max_frame-1])
                ymin=0.01

                plot,D[0:max_frame-1]+1,std[collect,3,iham,0,isubset,0:max_frame-1],xtit='frame',ytit='Standard deviation',xrange=[minframe[collect],maxframe[collect]], $
                  yrange=[ymin,ymax],title=title2,charsize=1.2,charthick=5,thick=5,/nodata,/ylog
            
                FOR idet=y, detectors_TEB[b]-1, 2 DO BEGIN
                    oplot,D[0:max_frame-1]+minframe[collect],std[collect,3,iham,idet,isubset,0:max_frame-1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                    oplot,D[0:max_frame-1]+minframe[collect],std[collect,3,iham,idet,isubset,0:max_frame-1],symsize=1,color=lcol[idet],thick=5
                ENDFOR
                oplot,D[0:max_frame-1]+minframe[collect],E[collect,odd_even,0:max_frame-1],symsize=1,color=lcol[1],thick=5

                XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                set_legend,1,chname,lcol,symbol
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Set low gain dn plotting environment.
;*****************************************************************************

close,/all

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outpath+'.low_dn.ps'
device,/landscape,filename=outfname,/color

D=findgen(4000)
HAM=['A','B']
GAIN=['Low gain','High gain']

;*****************************************************************************
;  Plot switch low gain dn vs scan.
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
    title='Switch low gain dn vs Scan '+band_name_TEB[3]+' Collect '+strtrim(string(col+1),2)
    FOR iham=0, nham-1 DO BEGIN
        FOR isubset=0, subsets_TEB[3]-1 DO BEGIN
            FOR j=0, 1 DO BEGIN
                IF mean(low_dn_frame[col,3,iham,0,isubset,0:max_scan-1,j]) EQ 0 THEN continue
                
                IF j EQ 0 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (high to low)'
                IF j EQ 1 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (low to high)'
                
                zz=WHERE(low_dn_frame[col,3,iham,0,isubset,0:max_scan-1,j] NE 0)
                
                ymax=max(low_dn_frame[col,3,iham,0:detectors_TEB[3]-1,isubset,zz,j],min=ymin)
                
                plot,D[0:max_scan/2-1]+1+zz[0],low_dn_frame[col,3,iham,0,isubset,zz,j],xtit='Scan',ytit='Switch low gain dn',xrange=[0,max_scan], $
                  yrange=[ymin,ymax],title=title2,charsize=1.2,charthick=5,thick=5,/nodata
                
                FOR idet=0, detectors_TEB[b]-1 DO BEGIN
                    oplot,2*D[0:max_scan/2-1]+1+zz[0],low_dn_frame[col,3,iham,idet,isubset,zz,j],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                    oplot,2*D[0:max_scan/2-1]+1+zz[0],low_dn_frame[col,3,iham,idet,isubset,zz,j],symsize=1,color=lcol[idet],thick=5
                ENDFOR
                XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
                set_legend,1,chname,lcol,symbol
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Plot switch low gain dn vs collect.
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

title='Switch low gain dn vs Collect '+band_name_TEB[3]
FOR iham=0, nham-1 DO BEGIN
    FOR isubset=0, subsets_TEB[3]-1 DO BEGIN
        FOR j=0, 1 DO BEGIN
            IF sig_mean_no_kick(low_dn_scan[0:ncollects-1,3,iham,0,isubset,j],1) LE 0 THEN continue

            IF j EQ 0 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (high to low)'
            IF j EQ 1 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (low to high)'
            
            ymax=max(low_dn_scan[0:ncollects-1,3,iham,0:detectors_TEB[3]-1,isubset,j],min=ymin)
            
            plot,D[0:ncollects-1]+1,low_dn_scan[0:ncollects-1,3,iham,0,isubset,j],xtit='Collect',ytit='Switch low gain dn',xrange=[0,ncollects], $
              yrange=[ymin,ymax],title=title2,charsize=1.2,charthick=5,thick=5,/nodata
            
            FOR idet=0, detectors_TEB[3]-1 DO BEGIN
                oplot,D[0:ncollects-1]+1,low_dn_scan[0:ncollects-1,3,iham,idet,isubset,j],symsize=1,color=lcol[idet],psym=symbol[idet],thick=5
                oplot,D[0:ncollects-1]+1,low_dn_scan[0:ncollects-1,3,iham,idet,isubset,j],symsize=1,color=lcol[idet],thick=5
            ENDFOR
            XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
            set_legend,1,chname,lcol,symbol
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Plot switch low gain radiance vs detector
;*****************************************************************************

!p.multi=[0,1,2]
!p.position=0

title='Gain Switching Radiance vs Detector '+band_name_TEB[3]
FOR iham=0, nham-1 DO BEGIN
    FOR isubset=0, subsets_TEB[3]-1 DO BEGIN
        FOR j=0, 1 DO BEGIN
            IF mean(low_temp[3,iham,0:detectors_TEB[b]-1,isubset,j]) LE 0 THEN continue
                
            IF j EQ 0 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (high to low)'
            IF j EQ 1 THEN title2='HAM side '+HAM[iham]+' SS'+strtrim(string(isubset+1),2)+' (low to high)'
                
            ymax=max(low_temp[3,iham,0:detectors_TEB[b]-1,isubset,j],min=ymin)
            IF ymax LT 350. THEN ymax=350.
            IF ymin GT 340. THEN ymin=340.
                
            plot,D[0:detectors_TEB[3]-1]+1,low_temp[3,iham,0:detectors_TEB[3]-1,isubset,j],xtit='Detector',ytit='Radiance (Wm-2sr-1um-1)', $
              yrange=[ymin,ymax],title=title2,charsize=1.2,charthick=5,thick=5,/nodata
                
            oplot,D[0:detectors_TEB[3]-1]+1,low_temp[3,iham,0:detectors_TEB[3]-1,isubset,j],symsize=1,color=lcol[0],psym=symbol[0],thick=5
            oplot,D[0:detectors_TEB[3]-1]+1,low_temp[3,iham,0:detectors_TEB[3]-1,isubset,j],symsize=1,color=lcol[0],thick=5
                
            oplot,[0.,40.],[343.,343.],symsize=1,color=lcol[1],linestyle=2,thick=5
            oplot,[0.,40.],[348.,348.],symsize=1,color=lcol[1],linestyle=2,thick=5
                
            XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
        ENDFOR
    ENDFOR
ENDFOR

device,/close

;*****************************************************************************
;  Write text files with max high gain dn, radiance, and temperature
;  as well as low gain switch dn, radiance, and temperature
;*****************************************************************************

outfile_txt1=outpath+'.max_dn.highlow.txt'
outfile_txt2=outpath+'.max_dn.lowhigh.txt'
outfile_txt1=outpath+'.low_dn.highlow.txt'
outfile_txt2=outpath+'.low_dn.lowhigh.txt'
fmt='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile_txt1,/get_lun
openw,lon,outfile_txt2,/get_lun
printf,lun,'Band   HAM   Detector   Max_dn    Max_L    Max_T'
printf,lon,'Band   HAM   Detector   Max_dn    Max_L    Max_T'
FOR ih=0, nham-1 DO BEGIN
    FOR id=0, detectors_TEB[3]-1 DO BEGIN
        FOR is=0, subsets_TEB[3]-1 DO BEGIN
            printf,lun,format=fmt,band_name_TEB[3],ih,id,max_dn[3,ih,id,is,0],max_L[3,ih,id,is,0],max_temp[3,ih,id,is,0]
            printf,lon,format=fmt,band_name_TEB[3],ih,id,max_dn[3,ih,id,is,1],max_L[3,ih,id,is,1],max_temp[3,ih,id,is,1]
            printf,lun,format=fmt,band_name_TEB[3],ih,id,low_dn[3,ih,id,is,0],low_L[3,ih,id,is,0],low_temp[3,ih,id,is,0]
            printf,lon,format=fmt,band_name_TEB[3],ih,id,low_dn[3,ih,id,is,1],low_L[3,ih,id,is,1],low_temp[3,ih,id,is,1]
        ENDFOR
    ENDFOR
ENDFOR
close,lun,/file,/all
free_lun,lun

;*****************************************************************************
;  Write a text file with any scans which have a gain switching anomaly
;*****************************************************************************

outfile_txt=outpath+'.high_low.high.txt'
fmt='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, d)'

openw,lun,outfile_txt,/get_lun
printf,lun,'High to Low gain transition anamolies (High gain)'
printf,lun,'Band   Collect   HAM   Detector   SCAN   Slope of transistion'
FOR col=0, ncollects-1 DO BEGIN
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[3]-1 DO BEGIN
            FOR iscan=0, max_scan-1 DO BEGIN
                IF transition_slope_up[col,3,iham,idet,0,iscan,0] EQ 0 THEN continue

                IF transition_slope_up[col,3,iham,idet,0,iscan,0] LT 500 THEN $
                  printf,lun,format=fmt,band_name_TEB[3],col,iham,idet,iscan,transition_slope_up[col,3,iham,idet,0,iscan,0]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
close,lun,/file,/all
free_lun,lun

outfile_txt=outpath+'.high_low.low.txt'

openw,lun,outfile_txt,/get_lun
printf,lun,'High to Low gain transition anamolies (Low gain)'
printf,lun,'Band   Collect   HAM   Detector   SCAN   Slope of transistion'
FOR col=0, ncollects-1 DO BEGIN
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[3]-1 DO BEGIN
            FOR iscan=0, max_scan-1 DO BEGIN
                IF transition_slope_up[col,3,iham,idet,0,iscan,1] EQ 0 THEN continue

                IF transition_slope_up[col,3,iham,idet,0,iscan,1] LT 500 THEN $
                  printf,lun,format=fmt,band_name_TEB[3],col,iham,idet,iscan,transition_slope_up[col,3,iham,idet,0,iscan,1]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
close,lun,/file,/all
free_lun,lun

outfile_txt=outpath+'.low_high.high.txt'

openw,lun,outfile_txt,/get_lun
printf,lun,'Low to High gain transition anamolies (High gain)'
printf,lun,'Band   Collect   HAM   Detector   SCAN   Slope of transistion'
FOR col=0, ncollects-1 DO BEGIN
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[3]-1 DO BEGIN
            FOR iscan=0, max_scan-1 DO BEGIN
                IF transition_slope_down[col,3,iham,idet,0,iscan,0] EQ 0 THEN continue

                IF transition_slope_down[col,3,iham,idet,0,iscan,0] GT -500 THEN $
                  printf,lun,format=fmt,band_name_TEB[3],col,iham,idet,iscan,transition_slope_down[col,3,iham,idet,0,iscan,0]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
close,lun,/file,/all
free_lun,lun

outfile_txt=outpath+'.low_high.low.txt'

openw,lun,outfile_txt,/get_lun
printf,lun,'Low to High gain transition anamolies (Low gain)'
printf,lun,'Band   Collect   HAM   Detector   SCAN   Slope of transistion'
FOR col=0, ncollects-1 DO BEGIN
    FOR iham=0, nham-1 DO BEGIN
        FOR idet=0, detectors_TEB[3]-1 DO BEGIN
            FOR iscan=0, max_scan-1 DO BEGIN
                IF transition_slope_down[col,3,iham,idet,0,iscan,1] EQ 0 THEN continue

                IF transition_slope_down[col,3,iham,idet,0,iscan,1] GT -500 THEN $
                  printf,lun,format=fmt,band_name_TEB[3],col,iham,idet,iscan,transition_slope_down[col,3,iham,idet,0,iscan,1]
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
close,lun,/file,/all
free_lun,lun

skipout:

END

